/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.EnumMap;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.util.NbBundle;

/**
 * FileProcessor for v/d Borne type Greenseeker sensor files.
 * @author Arjan Loermans
 */
public class GreenseekerFileProcessorBorne extends GreenseekerFileProcessor {

    public GreenseekerFileProcessorBorne(GreenseekerDataObject toProcess) {
        super(toProcess);
    }
    // Constant int values
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(GreenseekerFileProcessorBorne.class, "sensor.greenseeker.fileprocessor.borne.log.displayname");
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("HHmmss.SS");
    private static final AppLogger LOG = AppLogFactory.getLogger(GreenseekerFileProcessorBorne.class);
    public static final int nrOfFields = 20;

    /**
     * Parses one record of a v/d Borne Greenseeker sensorfile.
     *
     * @param payload The record to be processed.
     * @param dataFile The SensorDataFile belonging to this sensor file.
     * @return A record represented as a Greenseeker sensordata object
     * @throws ParseException Exception occurs when date/time cannot be parsed.
     */
    @Override
    protected GreenseekerSensorData parseLine(Object[] payload) throws GreenseekerParseException {

        EnumMap map = new EnumMap<GreenseekerFileDescriptionBorne, Object>(GreenseekerFileDescriptionBorne.class);
        try {
            for (GreenseekerFileDescriptionBorne greenseekerBorneField : GreenseekerFileDescriptionBorne.values()) {
                if (greenseekerBorneField.isMandatory()) {
                    map.put(greenseekerBorneField, payload[greenseekerBorneField.getPosition()]);
                }
            }
        } catch (Exception e) {
            throw new GreenseekerParseException(e);
        }

        GreenseekerSensorData sensorData = new GreenseekerSensorData();
        sensorData.setObjectId((Double) map.get(GreenseekerFileDescriptionBorne.OBJECTID));
        sensorData.setLongitude((Double) map.get(GreenseekerFileDescriptionBorne.LONGITUDE));
        sensorData.setLatitude((Double) map.get(GreenseekerFileDescriptionBorne.LATITUDE));
        sensorData.setSpeed((Double) map.get(GreenseekerFileDescriptionBorne.SPEED));
        sensorData.setHeading((Double) map.get(GreenseekerFileDescriptionBorne.HEADING));
        sensorData.setElevation((Double) map.get(GreenseekerFileDescriptionBorne.ELEVATION));

        sensorData.setNdvi((Double) map.get(GreenseekerFileDescriptionBorne.NDVI));
        sensorData.setVi2nd((Double) map.get(GreenseekerFileDescriptionBorne.VI2ND));

        sensorData.setRed54((Double) map.get(GreenseekerFileDescriptionBorne.RED54));
        sensorData.setNir54((Double) map.get(GreenseekerFileDescriptionBorne.NIR54));

        sensorData.setRed55((Double) map.get(GreenseekerFileDescriptionBorne.RED55));
        sensorData.setNir55((Double) map.get(GreenseekerFileDescriptionBorne.NIR55));

        sensorData.setRed57((Double) map.get(GreenseekerFileDescriptionBorne.RED57));
        sensorData.setNir57((Double) map.get(GreenseekerFileDescriptionBorne.NIR57));

        sensorData.setRed53((Double) map.get(GreenseekerFileDescriptionBorne.RED53));
        sensorData.setNir53((Double) map.get(GreenseekerFileDescriptionBorne.NIR53));

        sensorData.setRed56((Double) map.get(GreenseekerFileDescriptionBorne.RED56));
        sensorData.setNir56((Double) map.get(GreenseekerFileDescriptionBorne.NIR56));
        sensorData.setRed58((Double) map.get(GreenseekerFileDescriptionBorne.RED58));
        sensorData.setNir58((Double) map.get(GreenseekerFileDescriptionBorne.NIR58));

        return sensorData;
    }

    /**
     * Concrete subclasses must override this method to return the displayname
     * of that sensorprocessor to be displayed in the log. For example: "ASC-340
     * Cropcircle"
     *
     * Primary use is for logging, for example: [Info] Start processing of
     * ASC-340 Cropcircle sensor file 010710AD.CSV
     *
     * @return
     */
    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }
}
